@@ -1,12 +1,11 @@
 from abc import ABC, abstractmethod
-from collections import Sequence
+from collections.abc import Sequence
 from numbers import Number
 from typing import List, Tuple, Mapping
 
 import numpy as np
 import pandas as pd
 import sklearn
-import xgboost
 
 
 class ShadowDecTree(ABC):
@@ -56,14 +55,11 @@ class ShadowDecTree(ABC):
 
         self.feature_names = feature_names
         self.target_name = target_name
-        self.class_names = class_names
-        # self.class_weight = self.get_class_weight()
         self.x_data = ShadowDecTree._get_x_data(x_data)
         self.y_data = ShadowDecTree._get_y_data(y_data)
-        # self.node_to_samples = self.get_node_samples()
         self.root, self.leaves, self.internal = self._get_tree_nodes()
-        if class_names:
-            self.class_names = self._get_class_names()
+        if self.is_classifier():
+            self.class_names = self._normalize_class_names(class_names)
 
     @abstractmethod
     def is_fit(self) -> bool:
@@ -135,6 +131,11 @@ class ShadowDecTree(ABC):
         """
         pass
 
+    @abstractmethod
+    def get_split_samples(self, id):
+        """Returns left and right split indexes from a node"""
+        pass
+
     @abstractmethod
     def get_node_nsamples(self, id):
         """Returns number of samples for a specific node id."""
@@ -244,7 +245,7 @@ class ShadowDecTree(ABC):
 
     @abstractmethod
     def get_min_samples_leaf(self) -> (int, float):
-        """Returns the minimum number of samples required to be at a leaf node."""
+        """Returns the minimum number of samples required to be at a leaf node, during node splitting"""
         pass
 
     @abstractmethod
@@ -252,6 +253,9 @@ class ShadowDecTree(ABC):
         """Return true if it should go to the left node child based on node split criterion and x value"""
         pass
 
+    def get_root_edge_labels(self):
+        pass
+
     def is_categorical_split(self, id) -> bool:
         """Checks if the node split is a categorical one.
 
@@ -288,7 +292,7 @@ class ShadowDecTree(ABC):
             # print(f"\tmax={np.max(height_of_bins):2.0f}, heights={list(height_of_bins)}, {len(height_of_bins)} bins")
         return node_heights
 
-    def predict(self, x: np.ndarray) -> Tuple[Number, List]:
+    def predict(self, x: np.ndarray, path_only: bool = False) -> Tuple[Number, List]:
         """
         Given an x - vector of features, return predicted class or value based upon this tree.
         Also return path from root to leaf as 2nd value in return tuple.
@@ -314,6 +318,8 @@ class ShadowDecTree(ABC):
 
         path = []
         leaf = walk(self.root, x, path)
+        if path_only:
+            return path
         return leaf.prediction(), path
 
     def tesselation(self):
@@ -392,14 +398,17 @@ class ShadowDecTree(ABC):
         index, leaf_sample_0, leaf_samples_1 = zip(*leaf_samples)
         return index, leaf_sample_0, leaf_samples_1
 
-    def _get_class_names(self):
+    def _normalize_class_names(self, class_names):
         if self.is_classifier():
-            if isinstance(self.class_names, dict):
-                return self.class_names
-            elif isinstance(self.class_names, Sequence):
-                return {i: n for i, n in enumerate(self.class_names)}
+            if class_names is None:
+                return {i: f"class {i}" for i in range(self.nclasses())}
+            if isinstance(class_names, dict):
+                return class_names
+            elif isinstance(class_names, Sequence):
+                return {i: n for i, n in enumerate(class_names)}
             else:
-                raise Exception(f"class_names must be dict or sequence, not {self.class_names.__class__.__name__}")
+                raise Exception(f"class_names must be dict or sequence, not {class_names.__class__.__name__}")
+        return None
 
     def _get_tree_nodes(self):
         # use locals not args to walk() for recursion speed in python
@@ -438,17 +447,34 @@ class ShadowDecTree(ABC):
 
     @staticmethod
     def get_shadow_tree(tree_model, x_data, y_data, feature_names, target_name, class_names=None, tree_index=None):
+        if hasattr(tree_model, 'get_booster'):
+            # scikit-learn wrappers XGBClassifier and XGBRegressor allow you to
+            # extract the underlying xgboost.core.Booster with the get_booster() method:
+            tree_model = tree_model.get_booster()
         if isinstance(tree_model, ShadowDecTree):
             return tree_model
         elif isinstance(tree_model, (sklearn.tree.DecisionTreeRegressor, sklearn.tree.DecisionTreeClassifier)):
             from dtreeviz.models import sklearn_decision_trees
             return sklearn_decision_trees.ShadowSKDTree(tree_model, x_data, y_data, feature_names,
                                                         target_name, class_names)
-        elif isinstance(tree_model, xgboost.core.Booster):
+        elif str(type(tree_model)).endswith("xgboost.core.Booster'>"):
             from dtreeviz.models import xgb_decision_tree
             return xgb_decision_tree.ShadowXGBDTree(tree_model, tree_index, x_data, y_data,
                                                     feature_names, target_name, class_names)
-        else: raise ValueError(f"Tree model must be in (DecisionTreeRegressor, DecisionTreeClassifier, xgboost.core.Booster, but was {tree_model.__class__.__name__}")
+        elif (str(type(tree_model)).endswith("pyspark.ml.classification.DecisionTreeClassificationModel'>") or
+              str(type(tree_model)).endswith("pyspark.ml.classification.DecisionTreeClassificationModel'>")):
+            from dtreeviz.models import spark_decision_tree
+            return spark_decision_tree.ShadowSparkTree(tree_model, x_data, y_data,
+                                                       feature_names, target_name, class_names)
+        elif "lightgbm.basic.Booster" in str(type(tree_model)):
+            from dtreeviz.models import lightgbm_decision_tree
+            return lightgbm_decision_tree.ShadowLightGBMTree(tree_model, tree_index, x_data, y_data,
+                                                             feature_names, target_name, class_names)
+        else:
+            raise ValueError(
+                f"Tree model must be in (DecisionTreeRegressor, DecisionTreeClassifier, "
+                "xgboost.core.Booster, lightgbm.basic.Booster, pyspark DecisionTreeClassificationModel or "
+                f"pyspark DecisionTreeClassificationModel) but you passed a {tree_model.__class__.__name__}!")
 
 
 class ShadowDecTreeNode():
@@ -525,12 +551,7 @@ class ShadowDecTreeNode():
     def split_samples(self) -> Tuple[np.ndarray, np.ndarray]:
         """Returns the list of indexes to the left and the right of the split value."""
 
-        samples = np.array(self.samples())
-        node_X_data = self.shadow_tree.x_data[samples, self.feature()]
-        split = self.split()
-        left = np.nonzero(node_X_data < split)[0]
-        right = np.nonzero(node_X_data >= split)[0]
-        return left, right
+        return self.shadow_tree.get_split_samples(self.id)
 
     def isleaf(self) -> bool:
         return self.left is None and self.right is None
